﻿# 4_update_folders.ps1 - Reads the file created in 3_prepare_folders.ps1.
#                        Renames all folders found in the file.
#
# Copyright (c) 2017 - 2021 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore administrator, default is "admin"
$username = "admin"

# If password is empty, an interactive prompt will ask for it
$password = ''

# The hostname of the MailStore Server, default is "localhost"
$hostname = "localhost"

# TCP Port of the Management API, API must be enabled in the MailStore Server Service Configuration, default is 8463
$port = 8463

# Set this to $true when untrusted certificates are used, default is $false
$acceptAllCertificates = $false

# File where the folder list is stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH "mailstore-folders.txt")

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

if ($password -ceq ''){
    $password = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Password for $username")))
}

function Has-Succeeded($result) {
    if ($result.statusCode -ne "succeeded") {
        Write-Host ("Status  " + $result.statusCode)
        Write-Host ("Message " + $result.error.message)
        Write-Host ("Details " + $result.error.details)
        return $false
    }
    return $true
}

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

if (-not (Test-Path $filename -PathType Leaf)){
    Write-Host ($filename + " does not exist.")
    My-Exit
}

Write-Host "Creating API client"
$msapiclient = New-MSApiClient -Username $username -Password $password -MailStoreServer $hostname -Port $port -IgnoreInvalidSSLCerts:$acceptAllCertificates

Write-Host ("Reading folders file " + $filename)
foreach ($line in (Get-Content $filename))
{
    # Skip lines starting with # and empty lines
    if ($line.StartsWith("#") -or -not $line) {
        Write-Host ("Skipping line '" + $line + "'")
        continue
    }
    
    $foldername_parts = $line.Split("=")

    # skip folders which contain at least one equals character
    if ($foldername_parts.count -gt 2) {
		Write-Host ("Source foldername or target foldername contains at least one equals character (=). Line: {0}. Skipping" -f $line)
		continue
	}

    $source_folder = $foldername_parts[0]
    $target_folder = $foldername_parts[1]

    # skip folders without source
    if (-not $source_folder){
        Write-Host ("Source foldername of '" + $target_folder + "' is empty. Skipping")
        continue
    }

    # skip folders without target
    if (-not $target_folder){
        Write-Host ("Target foldername of '" + $source_folder + "' is empty. Skipping folder")
        continue
    }    
    
    # skip folders that do not change
    if ($source_folder -ceq $target_folder){
        Write-Host ("Old folder name and new folder name must not match, but they do: " + $source_folder + " = " + $target_folder)
        continue
    }

    Write-Host ("Moving folder " + $source_folder + " to " + $target_folder)
    $rename_folder_result = Invoke-MSApiCall $msapiclient "MoveFolder" @{fromFolder = $source_folder; toFolder = $target_folder}
    
    if (-not (Has-Succeeded($rename_folder_result))) {
        continue
    }
    # Write-Host $rename_folder_result.statusCode
}

My-Exit
